clear  % Cancella tutte le variabili presenti nello workspace
Pmax=620;  % Potenza massima del modulo fotovoltaico (620 W)
Vmp=44.55;  % Tensione al punto di massima potenza (44.55 V)
Imp=13.92;  % Corrente al punto di massima potenza (13.92 A)
Voc_stc=52.66;  % Tensione a circuito aperto in condizioni STC (52.66 V)
Isc_stc=14.81;  % Corrente di cortocircuito in condizioni STC (14.81 A)
eta=0.23;  % Efficienza del modulo (23%)
etaVoc=-0.23/100;  % Coefficiente di temperatura della tensione a circuito aperto (-0.23%/°C)
etaIsc=0.045/100;  % Coefficiente di temperatura della corrente di cortocircuito (0.045%/°C)
etaPmax=-0.28/100;  % Coefficiente di temperatura della potenza massima (-0.28%/°C)
T_STC=25;  % Temperatura di riferimento (25°C)
G_STC=1000;  % Irraggiamento di riferimento (1000 W/m²)
Rs=8*10^(-3);  % Resistenza serie del modulo (36 mΩ)

G=[200, 400, 600, 800, 1000];  % Vettore di irraggiamento da 200 a 1000 W/m² con 5 valori
DeltaI=zeros(length(G));  % Inizializza un vettore per le variazioni di corrente
DeltaV=zeros(length(G));  % Inizializza un vettore per le variazioni di tensione
Isc=zeros(length(G));  % Inizializza un vettore per le correnti di cortocircuito
Voc=zeros(length(G));  % Inizializza un vettore per le tensioni a circuito aperto


C2=(Vmp-Voc_stc)/log(1-Imp/Isc_stc);  % Calcola il parametro C2 per il modello esponenziale
C1=Isc_stc/(1-exp(-Voc_stc/C2));  % Calcola il parametro C1 per il modello esponenziale


% Aggiornamento di Isc e Voc in base alla variazione di irraggiamento (G)
for i=1:length(G)
    DeltaI(i)=Isc_stc*(G(i)/G_STC-1)+etaIsc*(G(i)/G_STC)*(T_STC-T_STC);  % Variazione di I_sc dovuta a G
    Isc(i)=Isc_stc+DeltaI(i);  % Nuova corrente di cortocircuito
    DeltaV(i)=etaVoc*(T_STC-T_STC)-Rs*DeltaI(i);  % Variazione di V_oc dovuta a G
    Voc(i)=Voc_stc+DeltaV(i);  % Nuova tensione a circuito aperto
end

V=linspace(0,55,5000);  % Vettore di tensioni da 0 a 55 V con 10000 punti
PV_I=real(zeros(length(V)+1,length(G)));  % Inizializza una matrice per le correnti PV


% Calcolo della corrente PV per ogni combinazione di tensione e irraggiamento
for i = 1:length(V)
    for j = 1:(length(G) + 1)
        if j == 1
            PV_I(i, j) = 0;  % Prima colonna della matrice è zero
        else 
            PV_I(i, j) = Isc(j - 1) - C1 * exp(-Voc(j - 1) / C2) * (exp(V(i) / C2) - 1);  % Modello esponenziale
            
            if PV_I(i, j) < 0
               PV_I(i, j) = NaN;  % Imposta a NaN se la corrente è negativa
            end
        end
    end
end

% Estrai l'ultima colonna di PV_I (corrispondente a G = 1000 W/m²)
I_G1000 = PV_I(1:5000, end);  % Usa solo i primi 5000 elementi per compatibilità con V
I_G1000 = I_G1000';  % Trasponi I_G1000 per renderlo un vettore riga (1x5000)

% Calcola la potenza P = V .* I_G1000 (prodotto elemento per elemento)
P = V .* I_G1000;  % P sarà un vettore riga 1x5000


% Plot delle curve I-V per diversi valori di irraggiamento (G)
I=Isc(1)-C1*exp(-Voc(1)/C2)*(exp(V/C2)-1);  % Corrente per il primo valore di G
 if I(I<0)<0 
    I(I<0) = NaN;  % Imposta a NaN se la corrente è negativa
 end
plot(V,I,'b');  % Plot della curva I-V per G=200 W/m² (blu)
hold on  % Mantieni il grafico attivo per sovrapporre le curve

I=Isc(2)-C1*exp(-Voc(2)/C2)*(exp(V/C2)-1);  % Corrente per il secondo valore di G
if  I(I<0)<0 
    I(I<0) = NaN;  % Imposta a NaN se la corrente è negativa
 end
plot(V,I,'r');  % Plot della curva I-V per G=400 W/m² (rosso)

I=Isc(3)-C1*exp(-Voc(3)/C2)*(exp(V/C2)-1);  % Corrente per il terzo valore di G
if I(I<0)<0 
   I(I<0) = NaN;  % Imposta a NaN se la corrente è negativa
 end
plot(V,I,'g');  % Plot della curva I-V per G=600 W/m² (verde)

I=Isc(4)-C1*exp(-Voc(4)/C2)*(exp(V/C2)-1);  % Corrente per il quarto valore di G
 if I(I<0)<0 
    I(I<0) = NaN;  % Imposta a NaN se la corrente è negativa
 end
plot(V,I,'y');  % Plot della curva I-V per G=800 W/m² (giallo)

I=Isc(5)-C1*exp(-Voc(5)/C2)*(exp(V/C2)-1);  % Corrente per il quinto valore di G
if I(I<0)<0 
   I(I<0) = NaN;  % Imposta a NaN se la corrente è negativa
 end
plot(V,I);  % Plot della curva I-V per G=1000 W/m² (colore predefinito)

I=Isc(6)-C1*exp(-Voc(6)/C2)*(exp(V/C2)-1);  % Corrente per il quinto valore di G
if  I(I<0)<0 
    I(I<0) = NaN;  % Imposta a NaN se la corrente è negativa
 end
plot(V,I);  % Plot della curva I-V per G=1000 W/m² (colore predefinito)

%plot(V, P, 'b', 'LineWidth', 2);  % Plot della curva P-V in blu separatamente

yyaxis left;  % Attiva l'asse y sinistro per la corrente
plot(V, I, 'r', 'LineWidth', 2);  % Curva I-V in rosso
ylabel('Corrente I [A]');  % Etichetta dell'asse y sinistro
ylim([0, 20]);  % Imposta i limiti dell'asse y sinistro

yyaxis right;  % Attiva l'asse y destro per la potenza
plot(V, P, 'b', 'LineWidth', 2);  % Curva P-V in blu
ylabel('Potenza P [W]');  % Etichetta dell'asse y destro 
ylim([0, max(P)*1.1]);  % Imposta i limiti dell'asse y destro 

% Aggiungi etichette, titolo e griglia
xlabel('Tensione V [V]');  % Etichetta dell'asse x
title('Curva I-V e P-V per G = 1000 W/m²');  % Titolo del grafico
grid on;  % Attiva la griglia
%legend('Curva I-V', 'Curva P-V');  % Legenda


% Trova il punto di massima potenza (MPP)
[etaPmax, idx] = max(P);  % Trova il valore massimo di P e il suo indice
Vmp = V(idx);  % Tensione al punto di massima potenza

% Aggiungi il punto MPP al grafico
plot(Vmp, etaPmax, 'ko', 'MarkerSize', 2.5, 'LineWidth', 2);  % Punto di massima potenza (MPP) commentare per vedere solo 
text(Vmp + 1, etaPmax - 5, sprintf('MPP: %.2f W', etaPmax), 'FontSize', 12);  % Annotazione MPP




